unit main;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, clTcpServer, clFtpServer, clUserMgr, SyncObjs, usersform;

type
  TMainForm = class(TForm)
    btnStart: TButton;
    btnStop: TButton;
    memLog: TMemo;
    edtPort: TEdit;
    Label1: TLabel;
    Label2: TLabel;
    clFtpServer1: TclFtpServer;
    Label3: TLabel;
    edtRootDir: TEdit;
    btnUsers: TButton;
    procedure clFtpServer1ReceiveCommand(Sender: TObject;
      AConnection: TclFtpCommandConnection; const ACommand, AText: String);
    procedure clFtpServer1SendReply(Sender: TObject;
      AConnection: TclFtpCommandConnection; const ACommand, AText: String);
    procedure clFtpServer1ServerError(Sender: TObject; AErrorCode: Integer;
      const AMessage: String);
    procedure clFtpServer1Start(Sender: TObject);
    procedure clFtpServer1Stop(Sender: TObject);
    procedure clFtpServer1AcceptConnection(Sender: TObject;
      AConnection: TclTcpConnection);
    procedure clFtpServer1Authenticate(Sender: TObject;
      Account: TclUserAccount; const APassword: String; var IsAuthorized,
      Handled: Boolean);
    procedure clFtpServer1CloseConnection(Sender: TObject;
      AConnection: TclTcpConnection);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure btnStartClick(Sender: TObject);
    procedure btnStopClick(Sender: TObject);
    procedure btnUsersClick(Sender: TObject);
  private
    FSynchronizer: TCriticalSection;
    procedure PutLogMessage(const ALogMessage: string);
  end;

var
  MainForm: TMainForm;

const
  cUserAccountFile = 'UserAccountFile.dat';

implementation

{$R *.dfm}

procedure TMainForm.clFtpServer1ReceiveCommand(Sender: TObject;
  AConnection: TclFtpCommandConnection; const ACommand, AText: String);
begin
  PutLogMessage('Command: ' + ACommand + ' ' + AText);
end;

procedure TMainForm.clFtpServer1SendReply(Sender: TObject;
  AConnection: TclFtpCommandConnection; const ACommand, AText: String);
begin
  PutLogMessage('Reply: ' + AText);
end;

procedure TMainForm.clFtpServer1ServerError(Sender: TObject;
  AErrorCode: Integer; const AMessage: String);
begin
  PutLogMessage('Server Error: ' + AMessage);
end;

procedure TMainForm.clFtpServer1Start(Sender: TObject);
begin
  PutLogMessage('=================='#13#10'Start Server');
end;

procedure TMainForm.clFtpServer1Stop(Sender: TObject);
begin
  PutLogMessage('Stop Server');
end;

procedure TMainForm.FormCreate(Sender: TObject);
var
  stream: TStream;
  R: TReader;
begin
  FSynchronizer := TCriticalSection.Create();
  if FileExists(cUserAccountFile) then
  begin
    stream := nil;
    R := nil;
    try
      stream := TFileStream.Create(cUserAccountFile, fmOpenRead);
      R := TReader.Create(stream, 2048);
      Assert(R.ReadValue() = vaCollection);
      R.ReadCollection(clFtpServer1.UserAccounts);
    finally
      R.Free();
      stream.Free();
    end;
  end;
end;

procedure TMainForm.FormDestroy(Sender: TObject);
var
  stream: TStream;
  W: TWriter;
begin
  stream := nil;
  W := nil;
  try
    stream := TFileStream.Create(cUserAccountFile, fmCreate);
    W := TWriter.Create(stream, 2048);
    W.WriteCollection(clFtpServer1.UserAccounts);
  finally
    W.Free();
    stream.Free();
  end;

  FSynchronizer.Free();
end;

procedure TMainForm.PutLogMessage(const ALogMessage: string);
begin
  if not (csDestroying in ComponentState) then
  begin
    FSynchronizer.Enter();
    try
      memLog.Lines.Add(ALogMessage);
    finally
      FSynchronizer.Leave();
    end;
  end;
end;

procedure TMainForm.btnStartClick(Sender: TObject);
begin
  clFtpServer1.Port := StrToInt(edtPort.Text);
  clFtpServer1.RootDir := edtRootDir.Text;
  clFtpServer1.Start();
end;

procedure TMainForm.btnStopClick(Sender: TObject);
begin
  clFtpServer1.Stop();
end;

procedure TMainForm.clFtpServer1AcceptConnection(Sender: TObject;
  AConnection: TclTcpConnection);
begin
  PutLogMessage('Accept Connection. Host: ' + AConnection.HostName);
end;

procedure TMainForm.clFtpServer1Authenticate(Sender: TObject;
  Account: TclUserAccount; const APassword: String; var IsAuthorized,
  Handled: Boolean);
begin
  IsAuthorized := (GetPasswordHash(APassword) = Account.Password);
  Handled := True;

  if IsAuthorized then
  begin
    PutLogMessage('Authenticate: ' + Account.UserName);
  end else
  begin
    PutLogMessage('Authenticate Failed: ' + Account.UserName);
  end;
end;

procedure TMainForm.clFtpServer1CloseConnection(Sender: TObject;
  AConnection: TclTcpConnection);
begin
  PutLogMessage('Close Connection. Host: ' + AConnection.HostName);
end;

procedure TMainForm.btnUsersClick(Sender: TObject);
begin
  TfrmUsers.EditUsers(clFtpServer1.UserAccounts);
end;

end.
