unit usersform;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, clUserMgr, clFtpServer, clCryptAPI, StdCtrls, ExtCtrls;

type
  TfrmUsers = class(TForm)
    lbUsers: TListBox;
    Label1: TLabel;
    btnAdd: TButton;
    btnDelete: TButton;
    Bevel1: TBevel;
    btnOk: TButton;
    btnCancel: TButton;
    Label2: TLabel;
    edtDisplayName: TEdit;
    Label3: TLabel;
    edtUserName: TEdit;
    Label4: TLabel;
    edtPassword: TEdit;
    Label5: TLabel;
    edtRootDir: TEdit;
    cbReadOnly: TCheckBox;
    procedure lbUsersClick(Sender: TObject);
    procedure btnAddClick(Sender: TObject);
    procedure btnDeleteClick(Sender: TObject);
    procedure edtDisplayNameChange(Sender: TObject);
    procedure edtUserNameChange(Sender: TObject);
    procedure edtPasswordChange(Sender: TObject);
    procedure edtRootDirChange(Sender: TObject);
    procedure cbReadOnlyClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
  private
    FLoading: Boolean;
    FAccounts: TclUserAccounts;
    procedure LoadData(Accounts: TclUserAccounts);
    procedure StoreData(Accounts: TclUserAccounts);
    procedure FillListbox(ASelectedIndex: Integer);
    function GetCurrentItem: TclFtpUserAccount;
    procedure UpdateControls;
    function GetListboxItemName(Account: TclUserAccount): string;
  public
    class procedure EditUsers(Accounts: TclUserAccounts);
  end;

function GetPasswordHash(const APassword: string): string;
  
implementation

{$R *.dfm}

function GetPasswordHash(const APassword: string): string;
var
  context: HCRYPTPROV;
  hash: HCRYPTHASH;
  data: PByte;
  hashSize, dwordSize: DWORD;
begin
  if not CryptAcquireContext(@context, nil, nil, PROV_RSA_SCHANNEL, 0) then
  begin
    if not CryptAcquireContext(@context, nil, nil, PROV_RSA_SCHANNEL, CRYPT_NEWKEYSET) then
    begin
      raise Exception.CreateFmt('CryptAcquireContext error %d', [GetLastError()]);
    end;
  end;
  try
    if not CryptCreateHash(context, CALG_MD5, 0, 0, @hash) then
    begin
      raise Exception.CreateFmt('CryptCreateHash error %d', [GetLastError()]);
    end;
    if not CryptHashData(hash, Pointer(APassword), Length(APassword), 0) then
    begin
      raise Exception.CreateFmt('CryptHashData error %d', [GetLastError()]);
    end;
    data := nil;
    try
      dwordSize := SizeOf(DWORD);
      if not CryptGetHashParam(hash, HP_HASHSIZE, @hashSize, @dwordSize, 0) then
      begin
        raise Exception.CreateFmt('CryptGetHashParam error %d', [GetLastError()]);
      end;
      GetMem(data, hashSize);
      if not CryptGetHashParam(hash, HP_HASHVAL, data, @hashSize, 0) then
      begin
        raise Exception.CreateFmt('CryptGetHashParam error %d', [GetLastError()]);
      end;
      SetLength(Result, hashSize);
      system.Move(data^, Pointer(Result)^, hashSize);
    finally
      FreeMem(data);
      CryptDestroyHash(hash);
    end;
  finally
    CryptReleaseContext(context, 0);
  end;
end;

{ TfrmUsers }

procedure TfrmUsers.LoadData(Accounts: TclUserAccounts);
var
  i: Integer;
begin
  FAccounts.Assign(Accounts);
  for i := 0 to FAccounts.Count - 1 do
  begin
    FAccounts[i].Password := '';
  end;
  FillListbox(0);
end;

procedure TfrmUsers.StoreData(Accounts: TclUserAccounts);
var
  i: Integer;
  item: TclUserAccount;
  psw: string;
begin
  for i := Accounts.Count - 1 downto 0 do
  begin
    if (FAccounts.AccountByUserName(Accounts[i].UserName) = nil) then
    begin
      Accounts.Delete(i);
    end;
  end;

  for i := 0 to FAccounts.Count - 1 do
  begin
    item := Accounts.AccountByUserName(FAccounts[i].UserName);
    if (item = nil) then
    begin
      item := Accounts.Add();
      item.Assign(FAccounts[i]);
      psw := GetPasswordHash(item.Password);
    end else
    begin
      psw := item.Password;
      item.Assign(FAccounts[i]);
      if (item.Password <> '') then
      begin
        psw := GetPasswordHash(item.Password);
      end;
    end;
    item.Password := psw;
  end;
end;

class procedure TfrmUsers.EditUsers(Accounts: TclUserAccounts);
var
  dlg: TfrmUsers;
begin
  dlg := TfrmUsers.Create(nil);
  try
    dlg.LoadData(Accounts);
    if (dlg.ShowModal() = mrOK) then
    begin
      dlg.StoreData(Accounts);
    end;
  finally
    dlg.Free();
  end;
end;

procedure TfrmUsers.lbUsersClick(Sender: TObject);
var
  item: TclFtpUserAccount;
begin
  item := GetCurrentItem();
  if (item = nil) then Exit;
  FLoading := True;
  try
    edtDisplayName.Text := item.DisplayName;
    edtUserName.Text := item.UserName;
    edtPassword.Text := item.Password;
    edtRootDir.Text := item.RootDir;
    cbReadOnly.Checked := item.ReadOnlyAccess;
  finally
    FLoading := False;
  end;
end;

procedure TfrmUsers.btnAddClick(Sender: TObject);
begin
  FAccounts.Add();
  FillListbox(FAccounts.Count - 1);
end;

procedure TfrmUsers.btnDeleteClick(Sender: TObject);
var
  ind: Integer;
begin
  ind := lbUsers.ItemIndex;
  if ind < 0 then Exit;
  FAccounts.Delete(ind);
  if (ind > 0) then
  begin
    Dec(ind);
  end;
  FillListbox(ind);
end;

procedure TfrmUsers.edtDisplayNameChange(Sender: TObject);
begin
  if FLoading then Exit;
  GetCurrentItem().DisplayName := edtDisplayName.Text;
  lbUsers.Items[lbUsers.ItemIndex] := GetListboxItemName(GetCurrentItem());
end;

procedure TfrmUsers.edtUserNameChange(Sender: TObject);
begin
  if FLoading then Exit;
  GetCurrentItem().UserName := edtUserName.Text;
  lbUsers.Items[lbUsers.ItemIndex] := GetListboxItemName(GetCurrentItem());
end;

procedure TfrmUsers.edtPasswordChange(Sender: TObject);
begin
  if FLoading then Exit;
  GetCurrentItem().Password := edtPassword.Text;
end;

procedure TfrmUsers.edtRootDirChange(Sender: TObject);
begin
  if FLoading then Exit;
  GetCurrentItem().RootDir := edtRootDir.Text;
end;

procedure TfrmUsers.cbReadOnlyClick(Sender: TObject);
begin
  if FLoading then Exit;
  GetCurrentItem().ReadOnlyAccess := cbReadOnly.Checked;
end;

procedure TfrmUsers.FormCreate(Sender: TObject);
begin
  FAccounts := TclUserAccounts.Create(nil, TclFtpUserAccount);
end;

procedure TfrmUsers.FormDestroy(Sender: TObject);
begin
  FAccounts.Free();
end;

procedure TfrmUsers.FillListbox(ASelectedIndex: Integer);
var
  i: Integer;
begin
  lbUsers.Items.Clear();
  for i := 0 to FAccounts.Count - 1 do
  begin
    lbUsers.Items.AddObject(GetListboxItemName(FAccounts[i]), FAccounts[i]);
  end;
  if (lbUsers.Items.Count > ASelectedIndex) then
  begin
    lbUsers.ItemIndex := ASelectedIndex;
  end;
  UpdateControls();
  lbUsersClick(nil);
end;

function TfrmUsers.GetCurrentItem: TclFtpUserAccount;
begin
  if (lbUsers.ItemIndex < 0) then
  begin
    Result := nil;
  end else
  begin
    Result := TclFtpUserAccount(lbUsers.Items.Objects[lbUsers.ItemIndex]);
  end;
end;

procedure TfrmUsers.UpdateControls;
var
  disable: Boolean;
begin
  disable := (lbUsers.Items.Count < 1);
  edtDisplayName.ReadOnly := disable;
  edtUserName.ReadOnly := disable;
  edtPassword.ReadOnly := disable;
  edtRootDir.ReadOnly := disable;
  cbReadOnly.Enabled := not disable;
end;

function TfrmUsers.GetListboxItemName(Account: TclUserAccount): string;
begin
  Result := Account.DisplayName;
  if (Result = '') then
  begin
    Result := Account.UserName;
  end;
  if (Result = '') then
  begin
    Result := 'Unknown';
  end;
end;

end.
